/*
    Connect four - written in java
    Copyright (C) 2020 Oliver Schappacher and Dorian Zedler

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package de.itsblue.ConnectFour.player;

import java.awt.event.*;

import de.itsblue.ConnectFour.Plate.*;
import de.itsblue.ConnectFour.*;

/**
 * LocalPlayer is a class meant for usage with de.itsblue.ConnectFour. It is
 * used for a player controlled by the local buttons in the button row.
 * 
 * @author Dorian Zedler
 */
public class LocalPlayer extends Player implements ActionListener {

    /**
     * Constructor
     * 
     * @param controlledByButtonRow The button row used to control the player.
     * @param usingPlateType        The type of plate the player is using.
     */
    public LocalPlayer(ButtonRow controlledByButtonRow, PlateType usingPlateType) {
        super(controlledByButtonRow, usingPlateType);
    }

    /**
     * Function to set wether it is this player's turn
     */
    @Override
    public void setIsMyTurn(boolean isMyTurn) {
        super.setIsMyTurn(isMyTurn);

        // Remove the action listener from the button row when inactive
        if (isMyTurn)
            this.gameControllingButtonRow.addActionListenerToButtons(this);
        else
            this.gameControllingButtonRow.removeActionListemerFromButtons(this);
    }

    /**
     * Catch the actions from the button row
     */
    @Override
    public void actionPerformed(ActionEvent e) {
        if (e.getSource() instanceof PlateInsertButton && this.isMyTurn) {
            // if the action source was a PlateInsertButton
            // -> trigger a plate insertion
            PlateInsertButton source = (PlateInsertButton) e.getSource();
            this.doMove(source.getIndex());
        }
    }

}